/**
 * 
 */
package de.ppi.psec.services;

import models.psec.Role;
import models.psec.User;


/**
 * Service which make the persistence for the psec-objects.  
 * @author niels
 *
 */
public interface PsecPersistanceService {
    /**
     * Finds a {@linkplain User} by his id.
     * @param userId the id of the {@linkplain User}
     * @return the {@linkplain User}
     */
    public User findUserById(Long userId);
    
    /**
     * Finds a {@linkplain User} by his login name, see {@linkplain User#getLoginname()}.
     * @param loginName the loginname of the {@linkplain User}
     * @return the {@linkplain User}
     */
    public User findUserByLoginname(String loginName);
    
    /**
     * Saves a {@linkplain User}.
     * @param user, a user-object which can be cast to the unterlying type.
     * @throws ClassCastException if the object is not from the type the 
     * service expected.
     */
    public void saveUser(User user);
    
    /**
     * Loaded the roles which matched to the uniquename.
     * @param rolename the unique Rolename.
     * @return the role which has the given name or null.
     */
    public Role findRoleByName(String rolename);

}
