/**
 * 
 */
package de.ppi.psec.services;


import java.util.List;

import models.psec.PsecRole;
import models.psec.PsecUser;
import models.psec.Role;
import models.psec.User;

import com.avaje.ebean.Ebean;


/**
 * PsecPersistanceService based on Ebean.
 * @author niels
 *
 */
public class PsecPersistanceServiceImpl implements PsecPersistanceService {
    
    private static PsecPersistanceService service;
    
    private final String ebeanServer;
    
    /**
     * Get the current persistance-service.
     * @return the current persistance-service.
     */
    public static PsecPersistanceService getService() {
        if (service == null) {
            service = new PsecPersistanceServiceImpl();
        }
        return service;
    }

    /**
     * Set the persistance-service. 
     * @param service the persistance-service.
     */
    public static void setService(PsecPersistanceService service) {
        PsecPersistanceServiceImpl.service = service;
    }

    

    
    /**
     * Creates a new service for teh Defaultebeanserver.
     */
    public PsecPersistanceServiceImpl() {
        this(null);
    }
    
    /**
     * Creates a new service.
     * @param ebeanServer the ebeanServer which should be used.
     */
    public PsecPersistanceServiceImpl(String ebeanServer) {
        super();
        this.ebeanServer = ebeanServer;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public User findUserById(Long userId) {
        return Ebean.getServer(ebeanServer).find(PsecUser.class, userId);
        
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public User findUserByLoginname(String loginName) {
        return Ebean.getServer(ebeanServer).find(PsecUser.class).
                where().eq("loginname", loginName).findUnique();
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void saveUser(User user) {
        Ebean.getServer(ebeanServer).save(user);
    }
    
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Role findRoleByName(String rolename) {
        final List<PsecRole> roles =  Ebean.getServer(ebeanServer).find(PsecRole.class).
                where().eq("byName", rolename).findList();
        if (roles.size() != 1) {
            if (roles.size() == 0) {
                return null;
            } else {
                throw new IllegalStateException("There exist not exactly 1 role to the unique-name");
            }
        }  else {
            return roles.get(0);
        }
    }


}
