package models.psec;

import java.io.UnsupportedEncodingException;
import java.security.Permission;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import play.Logger;
import play.data.validation.Constraints.Email;
import play.data.validation.Constraints.Required;
import play.db.ebean.Model;
import de.ppi.util.PasswordTools;

@Entity
public class PsecUser extends Model implements User {
	
    @Id
    @GeneratedValue
    private Long id;

    @Column(unique = true, nullable = false)
    //TODO niels @Unique
    @Required
    private String loginname = "";
    @Email
  //TODO niels @Unique
    @Column(unique = true)
    private String emailAddress;
    private String lastname = "";
    private String firstname = "";
    private String description = "";
    @Column(nullable = false)
    private boolean enabled = false;
    private int loginFailedCount = 0;
    @Temporal(TemporalType.TIMESTAMP)
    private Date expiredDate;

    /** The passwordhash */
    @Required
    private String password;

    @ManyToMany()
    private List<PsecRole> psecRoles;

    @ManyToMany()
    private List<PsecPermission> psecPermissions;

    @SuppressWarnings("unused")
    @Temporal(TemporalType.TIMESTAMP)
    private Date lastUpdate;

    
    //TODO niels das funktioniert nicht so bei EBean.
    //TODO niels JOB schreiben.
    @PreUpdate
    @PrePersist
    protected void onUpdate() {
        lastUpdate = new Date();
    }


    public List <Permission> getPermissions() {
        final List<Permission> permissions = new ArrayList <Permission>(psecPermissions.size());
        for (PsecPermission permission : psecPermissions) {
            permissions.add(permission.convertToPermission());
        }
        return permissions;
    }
    
    public List <String> getRoles() {
        final List<String> roleNames = new ArrayList <String>(psecRoles.size());
        for (PsecRole role : psecRoles) {
            roleNames.add(role.getName());
        }
        return roleNames;
    }
    
    
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }
    
    /**
     * 
     *{@inheritDoc}
     */
    public List <PsecRole> getPsecRoles() {
        return psecRoles;
    }

    
    /**
     * 
     *{@inheritDoc}
     */
    public void setPsecRoles(List <PsecRole> roles) {
        this.psecRoles = roles;
    }


    /**
     * 
     *{@inheritDoc}
     */
    public void addToRoles(PsecRole permsecRole) {
        if (psecRoles == null) {
            psecRoles = new ArrayList <PsecRole>();
        }
        psecRoles.add(permsecRole);
    }

    public List <PsecPermission> getPsecPermissions() {
        return psecPermissions;
    }


    public void setPsecPermissions(List <PsecPermission> permissions) {
        this.psecPermissions = permissions;
    }

    
    /**
     * 
     *{@inheritDoc}
     */
    public String getLoginname() {
        return loginname;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public void setLoginname(String loginname) {
        this.loginname = loginname;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public String getEmailAddress() {
        return emailAddress;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public void setEmailAddress(String emailAddress) {
        if (emailAddress == null || emailAddress.trim().length() == 0) {
            this.emailAddress = null;
        } else {
            this.emailAddress = emailAddress;
        }
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public String getLastname() {
        return lastname;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public void setLastname(String lastname) {
        this.lastname = lastname;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public String getFirstname() {
        return firstname;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public void setFirstname(String firstname) {
        this.firstname = firstname;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public String getDescription() {
        return description;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public void setDescription(String description) {
        this.description = description;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public int getLoginFailedCount() {
        return loginFailedCount;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public void setLoginFailedCount(int loginFailedCount) {
        this.loginFailedCount = loginFailedCount;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public boolean isEnabled() {
        return enabled;
    }


    
    /**
     * 
     *{@inheritDoc}
     */
    public String getPasswordHash() {
        return password;
    }


    /**
     * 
     *{@inheritDoc}
     */
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
        if (enabled) {
            expiredDate = null;
        }
    }

    /**
     * {@inheritDoc}
     */
    public void setPassword(String newPassword) {
        if (newPassword != null && newPassword.equals(this.password)) {
            //Prevent that the hash will be hashed again.
            return;
        }
        if (newPassword  != null && newPassword.length() > 0) {
            if (newPassword.length() >= 6) {
                this.password = PasswordTools.saltPasswordBase64(newPassword);
            } else {
                //TODO niels Validation.current().addError("password", "validation.minSize", "6");
            }
        } else {
        	//TODO niels Validation.current().addError("password", "validation.required");
        }
    }

    /**
     * 
     *{@inheritDoc}
     */
    public boolean checkPassword(String givenPassword) {
        try {
            return PasswordTools.checkDigestBase64(givenPassword,
                    this.password);
        } catch (UnsupportedEncodingException e) {
        	Logger.error("Can't encode " + this.password + ".", e);
        }
        return false;
    }

    /**
     * 
     *{@inheritDoc}
     */
    public Date getExpiredDate() {
        return expiredDate;
    }


    /**
     * 
     *{@inheritDoc}
     */
    public void setExpiredDate(Date expiredDate) {
        this.expiredDate = expiredDate;
    }


    /**
     * 
     *{@inheritDoc}
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((emailAddress == null) ? 0 : emailAddress.hashCode());
        result = prime * result
                + ((lastname == null) ? 0 : lastname.hashCode());
        return result;
    }

    /**
     * 
     *{@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        PsecUser other = (PsecUser) obj;
        if (emailAddress == null) {
            if (other.emailAddress != null) {
                return false;
            }
        } else if (!emailAddress.equals(other.emailAddress)) {
            return false;
        }
        if (lastname == null) {
            if (other.lastname != null) {
                return false;
            }
        } else if (!lastname.equals(other.lastname)) {
            return false;
        }
        return true;
    }


}
