/**
 *
 */
package de.ppi.psec.services;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.security.Permission;
import java.util.ArrayList;
import java.util.List;

import models.psec.PsecPermission;
import models.psec.PsecRole;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ppi.psec.permissions.PsecBasicPermission;

/**
 * @author niels
 *
 */
public class AuthorizationServiceTest {

    AuthorizationService service;

    AuthenticationServiceStub authServiceStub;

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        
    }


    @Before
    public void setUp() throws Exception {
        authServiceStub = new AuthenticationServiceStub();
        service = new AuthorizationServiceImpl(authServiceStub, true);
    }

    /**
     * Test method for {@link de.ppi.psec.services.AuthorizationService#hasRole(java.lang.String)}.
     */
    @Test
    public void testHasRole() {
        List<String> roles = new ArrayList<String>();
        roles.add("TESTER");
        roles.add("DEVELOPER");
        authServiceStub.setRoles(roles);
        assertFalse(service.hasRole("foo"));
        assertFalse(service.hasRole("TEST"));
        assertFalse(service.hasRole("Tester"));
        assertTrue(service.hasRole("TESTER"));
        assertTrue(service.hasRole("DEVELOPER"));
    }

    /**
     * Test method for {@link de.ppi.psec.services.AuthorizationService#hasPermission(java.security.Permission)}.
     */
    @Test
    public void testHasPermissionPermission() {
        List<Permission> permissions = new ArrayList<Permission>();
        permissions.add(new PsecBasicPermission("foo", "bar, git"));
        permissions.add(new PsecBasicPermission("good", "bzr, hg"));
        authServiceStub.setRoles(new ArrayList <String>());
        authServiceStub.setPermissions(permissions);
        assertFalse(service.hasPermission(new PsecBasicPermission("*", "*")));
        assertFalse(service.hasPermission(new PsecBasicPermission("foo", "*")));
        assertFalse(service.hasPermission(new PsecBasicPermission("*", "bzr, hg")));
        assertTrue(service.hasPermission(new PsecBasicPermission("foo", "bar")));
        assertTrue(service.hasPermission(new PsecBasicPermission("foo", "git")));
        permissions.add(new PsecBasicPermission("*", "*"));
        assertTrue(service.hasPermission(new PsecBasicPermission("*", "*")));
        assertTrue(service.hasPermission(new PsecBasicPermission("foo", "*")));
        assertTrue(service.hasPermission(new PsecBasicPermission("*", "bzr, hg")));
    }

    /**
     * Test method for {@link de.ppi.psec.services.AuthorizationService#hasPermission(java.lang.String, java.lang.String)}.
     */
    @Test
    public void testHasPermissionStringString() {
        List<Permission> permissions = new ArrayList<Permission>();
        permissions.add(new PsecBasicPermission("foo", "bar, git"));
        permissions.add(new PsecBasicPermission("good", "bzr, hg"));
        authServiceStub.setRoles(new ArrayList <String>());
        authServiceStub.setPermissions(permissions);
        assertFalse(service.hasPermission("*", "*"));
        assertFalse(service.hasPermission("foo", "*"));
        assertFalse(service.hasPermission("*", "bzr, hg"));
        assertTrue(service.hasPermission("foo", "bar"));
        assertTrue(service.hasPermission("foo", "git"));
        permissions.add(new PsecBasicPermission("*", "*"));
        assertTrue(service.hasPermission("*", "*"));
        assertTrue(service.hasPermission("foo", "*"));
        assertTrue(service.hasPermission("*", "bzr, hg"));
    }
    
    /**
     * Test method for {@link de.ppi.psec.services.AuthorizationService#hasPermission(java.lang.String, java.lang.String)}.
     */
    @Test
    public void testWithRoleHasPermissionStringString() {
        final AuthorizationServiceStub serviceStub = new AuthorizationServiceStub(authServiceStub);
        final PsecRole role = new PsecRole();
        final List<PsecPermission> permissions = new ArrayList<PsecPermission>();
        permissions.add(PsecPermission.createFromBasicPermission("Perm1", 
                new PsecBasicPermission("foo", "bar, git")));
        permissions.add(PsecPermission.createFromBasicPermission("Perm2",
        		new PsecBasicPermission("good", "bzr, hg")));
        role.setName("ROLE1");
        role.setPsecPermissions(permissions);
        serviceStub.addRole(role);
        final List<String> rolenames = new ArrayList <String>();
        rolenames.add("ROLE1");
        authServiceStub.setRoles(rolenames);
        List<Permission> directPermissions = new ArrayList<Permission>();
        directPermissions.add(new PsecBasicPermission("unknown", "unknown")); 
        authServiceStub.setPermissions(directPermissions);
        
        assertFalse(serviceStub.hasPermission("*", "*"));
        assertFalse(serviceStub.hasPermission("foo", "*"));
        assertFalse(serviceStub.hasPermission("*", "bzr, hg"));
        assertTrue(serviceStub.hasPermission("foo", "bar"));
        assertTrue(serviceStub.hasPermission("foo", "git"));
    }
    
}
